package QARLA;

# ------------------------------------------------------------------------

#Copyright (C) 2005 Enrique Amig and Jess Gimnez

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License as published by the Free Software Foundation; either
#version 2.1 of the License, or (at your option) any later version.

#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public
#License along with this library; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ------------------------------------------------------------------------

use strict;
use Data::Dumper;
use IQ::METRICS;
use IQ::IQXML;

sub start {
   #description _ given a set of metrics, systems and human references reads similarities
   #              for a given set of segments.
   #param1  _ set of metrics
   #param2  _ set of segments (topics)
   #param3  _ set of systems
   #param4  _ set of references (models)
   #param5  _ verbosity (0/1)
   #@return _ IQ structure

   my $M = shift;
   my $T = shift;
   my $S = shift;
   my $R = shift;
   my $verbose = shift;

   if ($verbose) { print STDERR "[INSIDE QARLA] READING SIMILARITIES... "; }
   my %IQ;

   my @topics;
   foreach my $t (@{$T}) { $topics[$t] = 1; }

   if ($verbose == 1) { print STDERR "["; }

   foreach my $m (@{$M}) {
      if ($verbose == 1) { print STDERR "$m.."; }
      foreach my $r1 (@{$R}) {
         foreach my $r2 (@{$R}) {
	    if ($r1 ne $r2) {
               IQXML::read_report($r1, $r2, $m, "$r1-$r2.$m.$METRICS::XMLEXT", \%IQ, \@topics, 'seg', $verbose);
	    }
         }
      }
      foreach my $s (@{$S}) {
         foreach my $r (@{$R}) {
            IQXML::read_report($s, $r, $m, "$s-$r.$m.$METRICS::XMLEXT", \%IQ, \@topics, 'seg', $verbose);
            #IQXML::read_report($r, $s, $m, "$r-$s.$m.$METRICS::XMLEXT", \%IQ, \@topics, 'seg', $verbose);
         }
      }
      #foreach my $s1 (@{$S}) {
      #   foreach my $s2 (@{$S}) {
      #      if ($s1 ne $s2) {
      #         IQXML::read_report($s1, $s2, $m, "$s1-$s2.$m.$METRICS::XMLEXT", \%IQ, \@topics, 'seg', $verbose);
      #      }
      #   }
      #}
   }

   if ($verbose == 1) { print STDERR "]\n"; }

   return \%IQ;
}

########################################
##### QUEEN
#######################################

sub QUEEN {
   #description _ compute QUEEN
   #param1  _ IQ structure
   #param2  _ set of metrics
   #param3  _ set of segments (topics)
   #param4  _ set of systems
   #param5  _ set of references (models)
   #param6  _ verbosity (0/1)

   my $IQ = shift;
   my $metrics = shift;
   my $topics = shift;
   my $peers = shift;
   my $models = shift;
   my $verbose = shift;

   #print Dumper($metrics);
   #print Dumper($topics);
   #print Dumper($peers);
   #print Dumper($models);

   my $Q=0;
   my $numTopics=0;
   foreach my $topic (@{$topics}) {
      $numTopics++;
      my $samples=0;
      my $Qtopic=0;
      foreach my $model0 (@{$models}) {	 
         foreach my $model1 (@{$models}){	 
	    foreach my $model2 (@{$models}){
 	       if ($model1 ne $model2){		     
	          foreach my $peer (@{$peers}){
	             if (($peer ne $model0) && ($peer ne $model1) && ($peer ne $model2)) { 	   
	 	        $samples++;			
                        my $existsMetric=0;
			foreach my $metric (@{$metrics}) {
			   if ($IQ->{$metric}->[$topic]->{$peer}->{$model0} <
			       $IQ->{$metric}->[$topic]->{$model1}->{$model2}) { $existsMetric = 1; }
		        }	       
		 	if ($existsMetric == 0) { $Qtopic++; }
		     }
		  }
	       }
	    }
	 }
      }
      if ($samples == 0) { $Qtopic = 0; }
      else { $Qtopic = $Qtopic / $samples; }
      $Q += $Qtopic;
   }

   if ($numTopics == 0) { return 0; }
   else { return $Q / $numTopics; }
}

########################################
##### IQ
#######################################

sub IQ {
   #description _ compute IQ (Innovated QUEEN)
   #param1  _ IQ structure
   #param2  _ set of metrics
   #param3  _ set of segments (topics)
   #param4  _ set of systems
   #param5  _ set of references (models)
   #param6  _ verbosity (0/1)

   my $IQ = shift;
   my $metrics = shift;
   my $topics = shift;
   my $peers = shift;
   my $models = shift;
   my $verbose = shift;

   #print Dumper($metrics);
   #print Dumper($topics);
   #print Dumper($peers);
   #print Dumper($models);

   my $Q=0;
   my $numTopics=0;
   foreach my $topic (@{$topics}) {
      $numTopics++;
      my $Qtopic=0;
      foreach my $model0 (@{$models}) {	 
         my $samplesMM=0;
         my $Qmodel=0;
         foreach my $model1 (@{$models}){	 
	    foreach my $model2 (@{$models}){
 	       if ($model1 ne $model2){		     
	          foreach my $peer (@{$peers}){
	             if (($peer ne $model0) && ($peer ne $model1) && ($peer ne $model2)) { 	   
                        my $existsMetric=0;
	 	        $samplesMM++;			
			foreach my $metric (@{$metrics}) {
			   if ($IQ->{$metric}->[$topic]->{$peer}->{$model0} <
			       $IQ->{$metric}->[$topic]->{$model1}->{$model2}) { $existsMetric = 1; }
		        }	       
		 	if ($existsMetric == 0) { $Qmodel++; }
		     }
		  }
	       }
	    }
	 }
         if ($samplesMM == 0) { $Qmodel = 0; }
         else { $Qmodel = $Qmodel / $samplesMM; }
         if ($Qmodel > $Qtopic) { $Qtopic = $Qmodel; }
      }
      if ($Qtopic == 1){ $Q++; }
   }

  $Q = $Q / $numTopics;
}


1;

